require 'code_128_test'

require 'code_25_test'
require 'code_25_interleaved_test'
require 'code_25_iata_test'

require 'code_39_test'
require 'code_93_test'

require 'codabar_test'

require 'ean13_test'
require 'ean8_test'
require 'bookland_test'
require 'upc_supplemental_test'

require 'data_matrix_test'
require 'qr_code_test'

#require 'pdf_417_test'
